;; Generated package description from tramp-nspawn.el  -*- no-byte-compile: t -*-
(define-package "tramp-nspawn" "1.0" "Tramp integration for systemd-nspawn containers" '((emacs "23")) :commit "b8380ee8a7c15ad4b96963d224969fc1a0337180" :authors '(("Brian Cully" . "bjc@kublai.com")) :maintainer '("Brian Cully" . "bjc@kublai.com") :keywords '("tramp" "nspawn" "machinectl" "systemd" "systemd-nspawn") :url "https://github.com/bjc/tramp-nspawn")
